==> fastec_3.2.3_43.tar.xz: usr/local/include/fastec/ffmpeg_container.h <==
/*!
 * @file FFmpegContainer.h
 * @brief Interface for coordinating access to and using the FFmpeg utility.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FFMPEG_CONTAINER_H
#define FFMPEG_CONTAINER_H
#pragma once

#include <string>
#include <memory>

#include "fas_video.h"
#include "fi_image.h"

#include "ffmpeg_status.h"

// This pulls out the information needed for AVPixelFormat usage.
#include "libavcodec/avcodec.h"

using namespace std::string_literals;

namespace FI_FFMPEG
{
//--------------------------------------------------------------------
// Constant, Typedefs and Macros
//--------------------------------------------------------------------
//#define DEBUG_FRAME_LEVEL_MSGS 1

//! Hardware Acceleration codec name is used to find the Vaapi codec.
//! There is no other ID available in FFmpeg source but the string is
//! used in the only example code found.
const std::string H264_VAAPI_CODEC_NAME = "h264_vaapi"s;

//! Hardware Acceleration using h264_qsv (Intel Quick Sync Video acceleration)
//! (H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10 (codec h264)
const std::string H264_INTEL_QSV_NAME       = "h264_qsv"s;
const std::string H264_INTEL_QSV_DEVICE     = "auto"s;   // Modify this with strings below to test MFX support.

//! Hardware Acceleration using h264_amf (AMD Media Framework acceleration)
//! (H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10 (codec h264)
const std::string H264_AMD_AMF_NAME         = "h264_amf"s;
const std::string H264_AMD_AMF_DEVICE       = "hw_any"s;   // see impl_map[] comment below for string options.
// Just modify this to AV_PIX_FMT_D3D11 or AV_PIX_FMT_DXVA2_VLD for testing.
const enum AVPixelFormat H264_AMD_AV_PIX_FMT = AV_PIX_FMT_D3D11;

//! Hardware Acceleration for Encoder h264_nvenc [NVIDIA NVENC H.264 encoder]
const std::string H264_NVIDIA_NVENC_NAME    = "h264_nvenc"s;
const std::string H264_NVIDIA_NVENC_NAME_2  = "nvenc_h264"s; // This is older not used anymore.

#if 0
// NOTE:  This "map" below comes from the FFmpeg hwcontext_qsv.c file inside of this
// Function:  static mfxIMPL choose_implementation(const char *device)
// It represents the string used for the H264_INTEL_QSV_DEVICE that will be used.
} impl_map[] =
{
    { "auto",     MFX_IMPL_AUTO         },
    { "sw",       MFX_IMPL_SOFTWARE     },
    { "hw",       MFX_IMPL_HARDWARE     },
    { "auto_any", MFX_IMPL_AUTO_ANY     },
    { "hw_any",   MFX_IMPL_HARDWARE_ANY },
    { "hw2",      MFX_IMPL_HARDWARE2    },
    { "hw3",      MFX_IMPL_HARDWARE3    },
    { "hw4",      MFX_IMPL_HARDWARE4    },
#endif

//! Type of FFMpeg container this context supports.
    typedef enum CONTAINER_TYPE
    {
        NO_CONTAINER,
        MP4_CONTAINER,
        //MXF_CONTAINER,    // NOTE_JLC Are we supporting this format?
    } CONTAINER_TYPE;

//! Type of operation this FFMpeg contaner supports.
    typedef enum CONTAINER_OPS
    {
        NO_OPERATION,   //!< Uninitialized for any operation
        TRANSCODER,     //!< Transcoder uses a decoder to pull frames to send to encoder.
        ENCODER,        //!< Encoder uses only an encoder and takes in frame information.
    } CONTAINER_OPS;

//--------------------------------------------------------------------
// Class Definitions
//--------------------------------------------------------------------

    class FFmpegEncoder;
    class FFmpegFilters;

    class FFmpegContainer
    {
    public:
        virtual ~FFmpegContainer();

        // This object, for now, is non-copyable and non-copy assignable.
        FFmpegContainer(const FFmpegContainer & i) = delete;
        FFmpegContainer& operator=(const FFmpegContainer & i) = delete;

        //! @{
        //! Each type of FFmpeg Container supported will have its own factory creation method.
        //!
        //! In order to create an FFmpeg container object the factory must be supplied
        //! with the video frame type of input and the video frame type of output.
        //!
        //! All the internal knowledge related to transcoding or encoding is based upon
        //! the type of video frame data that will be streamed to the container.
        static std::shared_ptr<FI_FFMPEG::FFmpegContainer> CreateMP4Container(const std::string & videoNameToCreate,
                                                                              std::shared_ptr<FAS_VIDEO::FiImage> firstVideoFrame,
                                                                              uint32_t frameRateToUse,
                                                                              uint32_t frameCountToUse,
                                                                              uint32_t crfValue,
                                                                              const char *presetString,
                                                                              uint32_t keyframeIntervalToUse,
                                                                              bool swOverrideFlag);

        static std::shared_ptr<FI_FFMPEG::FFmpegContainer> CreateProResContainer(std::shared_ptr<FAS_VIDEO::FiImage> firstVideoFrame);
        //! @}

        static AVPixelFormat GetFFmpegPixelFormat(uint32_t FIFourCC);
        static const char *GetStringForAVPixelFormat(enum AVPixelFormat avPixelFormatId);
        static const char *GetStringForAVCodecID(AVCodecID avCodecID);

        static void LogAVFrameInfo(AVFrame * avFrameToLog, const char *frameNameStr = nullptr, bool onlyShowPictTypeInfo = false);
        static void log_callback_report(void *ptr, int level, const char *fmt, va_list vl);

        // Used to determine if the FFmpeg stream is valid for use.
        bool isFFmpegStreamOK();

        bool CloseDown();

        bool EncodeFrame(std::shared_ptr<FAS_VIDEO::FiImage>videoFrameToEncode);
        bool EncodeLostFrameCondition();

        uint32_t GetFramesAdded();
        uint32_t GetEncodedFrameCount(bool & isPrepared, bool & isComplete, bool & hasError);
        void GetStatusInfo(std::string & statusInfoStr);

        bool IsFrameWriterComplete(bool & inError);
        bool IsVideoComplete(uint32_t& nFramesToEncodeStill, bool & inError);

    protected:
        FFmpegContainer();

        bool PrepareInternalObjects();

        bool IncrementFramesProcessed();

        std::string videoNameForContainer;  //!< The name of the video being built by this container.
        CONTAINER_TYPE containerType;       //!< Container type.
        CONTAINER_OPS containerOp;          //!< Container operation.

        FFmpegStatus containerStatus;       //!< Container status.

        std::shared_ptr<FAS_VIDEO::FiImage> videoFrameToProcess;    //!< Holds video frame for FFMpeg processing.

        FAS_VIDEO::IMAGE_FORMAT inFIVideoFrameFormat;               //!< Input video frame Fastec Imaging format.
        enum AVPixelFormat hwFFmpegDeviceFmt;                       //!< HW FFmpeg device pix_fmt value if one exists.
        enum AVPixelFormat inFFMpegVideoFrameFmt;                   //!< Input video frame ffmpeg AVPixelFormat.
        enum AVPixelFormat outFFMpegVideoFrameFmt;                  //!< Output video frame ffmpeg AVPixelFormat.

        //! How many frames are expected in the container.
        uint32_t frameCount;

        //! Number of frames to render per second of video time.
        uint32_t frameRate;

        //! Number of bytes in a full frame of data.
        uint32_t fullFrameSize;

        //! How many frames have been processed when encoding.
        uint32_t framesProcessed;

        //! The number of placeholder frames requested in a row.
        uint32_t framesLost;

        //! How many placeholder frames pushed through the system.
        uint32_t totalFramesLost;

        //! The Constant Rate Factor option setting for H.264 encoding.
        uint32_t crfValue;

        //! The Preset string setting.
        std::string presetSetting;

        //! The Keyframe Interval setting.
        uint32_t keyframeInterval;

        //! The container was closed down.
        bool containerIsClosed;
    };

}
#endif // FFMPEG_CONTAINER_H

==> fastec_3.4.9.tar.xz: usr/local/include/fastec/ffmpeg_container.h <==
/*!
 * @file FFmpegContainer.h
 * @brief Interface for coordinating access to and using the FFmpeg utility.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FFMPEG_CONTAINER_H
#define FFMPEG_CONTAINER_H
#pragma once

#include <string>
#include <memory>

#include "fas_video.h"
#include "fi_image.h"

#include "ffmpeg_status.h"

// This pulls out the information needed for AVPixelFormat usage.
#include "libavcodec/avcodec.h"

using namespace std::string_literals;

namespace FI_FFMPEG
{
//--------------------------------------------------------------------
// Constant, Typedefs and Macros
//--------------------------------------------------------------------
//#define DEBUG_FRAME_LEVEL_MSGS 1

//! Hardware Acceleration codec name is used to find the Vaapi codec.
//! There is no other ID available in FFmpeg source but the string is
//! used in the only example code found.
const std::string H264_VAAPI_CODEC_NAME = "h264_vaapi"s;

//! Hardware Acceleration using h264_qsv (Intel Quick Sync Video acceleration)
//! (H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10 (codec h264)
const std::string H264_INTEL_QSV_NAME       = "h264_qsv"s;
const std::string H264_INTEL_QSV_DEVICE     = "auto"s;   // Modify this with strings below to test MFX support.

//! Hardware Acceleration using h264_amf (AMD Media Framework acceleration)
//! (H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10 (codec h264)
const std::string H264_AMD_AMF_NAME         = "h264_amf"s;
const std::string H264_AMD_AMF_DEVICE       = "hw_any"s;   // see impl_map[] comment below for string options.
// Just modify this to AV_PIX_FMT_D3D11 or AV_PIX_FMT_DXVA2_VLD for testing.
const enum AVPixelFormat H264_AMD_AV_PIX_FMT = AV_PIX_FMT_D3D11;

//! Hardware Acceleration for Encoder h264_nvenc [NVIDIA NVENC H.264 encoder]
const std::string H264_NVIDIA_NVENC_NAME    = "h264_nvenc"s;
const std::string H264_NVIDIA_NVENC_NAME_2  = "nvenc_h264"s; // This is older not used anymore.

#if 0
// NOTE:  This "map" below comes from the FFmpeg hwcontext_qsv.c file inside of this
// Function:  static mfxIMPL choose_implementation(const char *device)
// It represents the string used for the H264_INTEL_QSV_DEVICE that will be used.
} impl_map[] =
{
    { "auto",     MFX_IMPL_AUTO         },
    { "sw",       MFX_IMPL_SOFTWARE     },
    { "hw",       MFX_IMPL_HARDWARE     },
    { "auto_any", MFX_IMPL_AUTO_ANY     },
    { "hw_any",   MFX_IMPL_HARDWARE_ANY },
    { "hw2",      MFX_IMPL_HARDWARE2    },
    { "hw3",      MFX_IMPL_HARDWARE3    },
    { "hw4",      MFX_IMPL_HARDWARE4    },
#endif

//! Type of FFMpeg container this context supports.
    typedef enum CONTAINER_TYPE
    {
        NO_CONTAINER,
        MP4_CONTAINER,
        //MXF_CONTAINER,    // NOTE_JLC Are we supporting this format?
    } CONTAINER_TYPE;

//! Type of operation this FFMpeg contaner supports.
    typedef enum CONTAINER_OPS
    {
        NO_OPERATION,   //!< Uninitialized for any operation
        TRANSCODER,     //!< Transcoder uses a decoder to pull frames to send to encoder.
        ENCODER,        //!< Encoder uses only an encoder and takes in frame information.
    } CONTAINER_OPS;

//--------------------------------------------------------------------
// Class Definitions
//--------------------------------------------------------------------

    class FFmpegEncoder;
    class FFmpegFilters;

    class FFmpegContainer
    {
    public:
        virtual ~FFmpegContainer();

        // This object, for now, is non-copyable and non-copy assignable.
        FFmpegContainer(const FFmpegContainer & i) = delete;
        FFmpegContainer& operator=(const FFmpegContainer & i) = delete;

        //! @{
        //! Each type of FFmpeg Container supported will have its own factory creation method.
        //!
        //! In order to create an FFmpeg container object the factory must be supplied
        //! with the video frame type of input and the video frame type of output.
        //!
        //! All the internal knowledge related to transcoding or encoding is based upon
        //! the type of video frame data that will be streamed to the container.
        static std::shared_ptr<FI_FFMPEG::FFmpegContainer> CreateMP4Container(const std::string & videoNameToCreate,
                                                                              std::shared_ptr<FAS_VIDEO::FiImage> firstVideoFrame,
                                                                              uint32_t frameRateToUse,
                                                                              uint32_t frameCountToUse,
                                                                              uint32_t crfValue,
                                                                              const char *presetString,
                                                                              uint32_t keyframeIntervalToUse,
                                                                              bool swOverrideFlag,
                                                                              uint32_t lostFrameCountInFront);

        static std::shared_ptr<FI_FFMPEG::FFmpegContainer> CreateProResContainer(std::shared_ptr<FAS_VIDEO::FiImage> firstVideoFrame);
        //! @}

        static AVPixelFormat GetFFmpegPixelFormat(uint32_t FIFourCC);
        static const char *GetStringForAVPixelFormat(enum AVPixelFormat avPixelFormatId);
        static const char *GetStringForAVCodecID(AVCodecID avCodecID);

        static void LogAVFrameInfo(AVFrame * avFrameToLog, const char *frameNameStr = nullptr, bool onlyShowPictTypeInfo = false);
        static void log_callback_report(void *ptr, int level, const char *fmt, va_list vl);

        // Used to determine if the FFmpeg stream is valid for use.
        bool isFFmpegStreamOK();

        bool CloseDown();

        bool EncodeFrame(std::shared_ptr<FAS_VIDEO::FiImage>videoFrameToEncode, uint32_t duplicateFrameCount);

        uint32_t GetFramesAdded();
        uint32_t GetEncodedFrameCount(bool & isPrepared, bool & isComplete, bool & hasError);
        void GetStatusInfo(std::string & statusInfoStr);

        bool IsFrameWriterComplete(bool & inError);
        bool IsVideoComplete(uint32_t& nFramesToEncodeStill, bool & inError);

        bool PadWithEmptyFramesAtEnd();

    protected:
        FFmpegContainer();

        bool IncrementFramesProcessed();

        bool PrepareInternalObjects(uint32_t framesLostInFront);

        std::string videoNameForContainer;  //!< The name of the video being built by this container.
        CONTAINER_TYPE containerType;       //!< Container type.
        CONTAINER_OPS containerOp;          //!< Container operation.

        FFmpegStatus containerStatus;       //!< Container status.

        std::shared_ptr<FAS_VIDEO::FiImage> videoFrameToProcess;    //!< Holds video frame for FFMpeg processing.

        FAS_VIDEO::IMAGE_FORMAT inFIVideoFrameFormat;               //!< Input video frame Fastec Imaging format.
        enum AVPixelFormat hwFFmpegDeviceFmt;                       //!< HW FFmpeg device pix_fmt value if one exists.
        enum AVPixelFormat inFFMpegVideoFrameFmt;                   //!< Input video frame ffmpeg AVPixelFormat.
        enum AVPixelFormat outFFMpegVideoFrameFmt;                  //!< Output video frame ffmpeg AVPixelFormat.

        //! How many frames are expected in the container.
        uint32_t frameCount;

        //! Number of frames to render per second of video time.
        uint32_t frameRate;

        //! Number of bytes in a full frame of data.
        uint32_t fullFrameSize;

        //! How many frames have been processed when encoding.
        uint32_t framesProcessed;

        //! How many placeholder frames pushed through the system.
        uint32_t totalFramesLost;

        //! The Constant Rate Factor option setting for H.264 encoding.
        uint32_t crfValue;

        //! The Preset string setting.
        std::string presetSetting;

        //! The Keyframe Interval setting.
        uint32_t keyframeInterval;

        //! The container was closed down.
        bool containerIsClosed;
    };

}
#endif // FFMPEG_CONTAINER_H
